'use strict';

console.log('b');

var b = /*#__PURE__*/Object.freeze({
	__proto__: null
});

console.log('c');

var c = /*#__PURE__*/Object.freeze({
	__proto__: null
});

exports.b = b;
exports.c = c;
