System.register(['./generated-shared2.js', './generated-dep1.js', './generated-dep2.js'], (function () {
	'use strict';
	return {
		setters: [function () {}, function () {}, function () {}],
		execute: (function () {



		})
	};
}));
