define(['exports', './answer.num', './lorem.str', './no-ext'], (function (exports, answer, lorem, noExt) { 'use strict';



	exports.answer = answer;
	exports.lorem = lorem;
	exports.noExt = noExt;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
