define(['exports'], (function (exports) { 'use strict';

	const dep = 'dep';

	console.log('lib', dep);

	var lib = /*#__PURE__*/Object.freeze({
		__proto__: null
	});

	exports.dep = dep;
	exports.lib = lib;

}));
