'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const renderStart = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/assets/renderStart-66600c78.txt').href : new URL('assets/renderStart-66600c78.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const renderStartNamed = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/renderStart.txt').href : new URL('renderStart.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const renderStartNamedImmediately = 'renderStart.txt';
const banner = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/assets/banner-2b65cc0c.txt').href : new URL('assets/banner-2b65cc0c.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const bannerNamed = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/banner.txt').href : new URL('banner.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const footer = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/assets/footer-e1d2fccb.txt').href : new URL('assets/footer-e1d2fccb.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const footerNamed = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/footer.txt').href : new URL('footer.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const intro = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/assets/intro-520a8116.txt').href : new URL('assets/intro-520a8116.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const introNamed = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/intro.txt').href : new URL('intro.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const outro = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/assets/outro-21f77720.txt').href : new URL('assets/outro-21f77720.txt', document.currentScript && document.currentScript.src || document.baseURI).href);
const outroNamed = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __dirname + '/outro.txt').href : new URL('outro.txt', document.currentScript && document.currentScript.src || document.baseURI).href);

exports.banner = banner;
exports.bannerNamed = bannerNamed;
exports.footer = footer;
exports.footerNamed = footerNamed;
exports.intro = intro;
exports.introNamed = introNamed;
exports.outro = outro;
exports.outroNamed = outroNamed;
exports.renderStart = renderStart;
exports.renderStartNamed = renderStartNamed;
exports.renderStartNamedImmediately = renderStartNamedImmediately;
