define(['exports', './generated-main'], (function (exports, main) { 'use strict';



	exports.component = main.component;
	exports.lib = main.lib.named.named;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
