define(['exports', './generated-lib1', './generated-lib2'], (function (exports, lib1, lib2) { 'use strict';



	exports.lib1 = lib1.lib1;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
