'use strict';

var hsl2hsv$1 = require('./hsl2hsv.js');

var hsl2hsv = 'asdf';

console.log(hsl2hsv);

var lib = /*#__PURE__*/Object.freeze({
	__proto__: null,
	hsl2hsv: hsl2hsv$1["default"]
});

exports.lib = lib;
