System.register('myBundle', [], (function (exports) {
	'use strict';
	return {
		execute: (function () {

			var dep$1 = exports('depJs', 'js');

			var dep = exports('depMjs', 'mjs');

		})
	};
}));
