'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var x = require('x');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var x__default = /*#__PURE__*/_interopDefaultLegacy(x);



Object.defineProperty(exports, 'x', {
	enumerable: true,
	get: function () { return x__default["default"]; }
});
