define(['exports', 'lib'], (function (exports, value) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var value__default = /*#__PURE__*/_interopDefaultLegacy(value);

	var dep = 2 * value__default["default"];

	exports.dep = dep;

}));
