define(['exports', './hsl2hsv', './generated-index'], (function (exports, hsl2hsv, index) { 'use strict';



	exports.hsl2hsv = hsl2hsv["default"];

	Object.defineProperty(exports, '__esModule', { value: true });

}));
