console.log('resolved');
console.log('resolved');
console.log('resolved');

console.log(import.meta.url);
console.log(import.meta.privateProp);
console.log(import.meta);

console.log('url=es.js:resolve-import-meta-url/main.js');
console.log('privateProp=es.js:resolve-import-meta-url/main.js');
console.log('null=es.js:resolve-import-meta-url/main.js');
