define(['exports'], (function (exports) { 'use strict';

	console.log('dep');

	console.log('main');

	const value = 42;

	exports.value = value;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
