'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const separate = Promise.resolve().then(function () { return require('./generated-separate.js'); });

exports.separate = separate;
