define(['exports'], (function (exports) { 'use strict';

	/** A comment for a number */
	var number = 5;

	/** A comment for obj */
	var obj = { number };

	exports.obj = obj;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
