export { default as fooFalse, barFalse } from 'external-false';
export { default as fooTrue, barTrue } from 'external-true';
export * as externalFalse from 'external-false';
export * as externalTrue from 'external-true';
