(function () {
  'use strict';

  // jsjjjsjjjjsjs

  /* jsjjddjksj */ // Too many comments lol

  function Fn

  //iian iaouns
  /* aaain ob skubz


  */

  () {
    console.log("Foo");
  }

  // jsjjjsjjjjsjs

  /* jsjjddjksj
  */ // Too many comments lol

  async /* [no LineTerminator here] */ function Async

  //iian iaouns
  /* aaain ob skubz */

  () {
    console.log("Foo");
  }

  // jsjjjsjjjjsjs

  /* jsjjddjksj */ // Too many comments lol

  function

  /* oiasnpiueno */
  /// iauianpns

  /* aiusni
  */

  * Generator

  //iian iaouns
  /* aaain ob skubz */

  () {
    console.log("Foo");
  }

  // jsjjjsjjjjsjs

  /* jsjjddjksj */ // Too many comments lol

  class Class

  /* oiasnpiueno */
  /// iauianpns

  /* aiusni
  */

  //iian iaouns
  /* aaain ob skubz */

  {
    constructor() {
      console.log("Foo");
    }
  }

  /* 2 */function spacelessFn/* 3 */()/* 4 */{
    console.log("Foo");
  }

  function* spacelessGenerator(){
    console.log("Foo");
  }

  Fn();
  Async();
  Generator();
  new Class();
  spacelessFn();
  spacelessGenerator();

})();
