define(['../custom_modules/@my-scope/my-base-pkg/index', '../_virtual/index.js_commonjs-exports'], (function (index$1, index) { 'use strict';

  const base2 = index.__exports;

  var module = {
    base2,
  };

  return module;

}));
