define(['exports', './components/index', './components/sub/index'], (function (exports, index, index$1) { 'use strict';



	exports.baz = index.baz;
	exports.foo = index$1.foo;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
