import Fn from "./fn.js"
import Async from "./async.js";
import Generator from "./generator.js";
import Class from "./class.js";
import spacelessFn from "./spacelessFn.js"
import spacelessGenerator from "./spacelessGenerator.js";

Fn();
Async();
Generator();
new Class();
spacelessFn();
spacelessGenerator();
