define(['exports', './module-a/v1/index', './module-b/v1/index'], (function (exports, index, index$1) { 'use strict';



	exports.ModuleA_V1 = index;
	exports.ModuleB_V1 = index$1;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
