'use strict';

var $ = require('jquery');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var $__default = /*#__PURE__*/_interopDefaultLegacy($);

$__default["default"]( function () {
	$__default["default"]( 'body' ).html( '<h1>hello world!</h1>' );
});
