System.register('stirred', [], (function () {
	'use strict';
	return {
		execute: (function () {

			const other$1 = {
				something: 'here'
			};

			const other = {
				somethingElse: 'here'
			};

		})
	};
}));
