'use strict';

var name = require('./generated-name.js');
var secondName = require('./generated-secondName.js');
var name$1 = require('./generated-name2.js');

console.log('main', name, secondName, name$1);
