System.register([], (function (exports, module) {
	'use strict';
	return {
		execute: (function () {

			const asset = exports('asset', 'chunkId=nested/chunk2.js:moduleId=solved:fileName=assets/asset-solved-28a7ac89.txt:format=system:relativePath=../assets/asset-solved-28a7ac89.txt:referenceId=6296c678');
			const chunk = exports('chunk', 'chunkId=nested/chunk2.js:moduleId=solved:fileName=nested/chunk.js:format=system:relativePath=chunk.js:referenceId=f6c25ae7');

		})
	};
}));
