const FONT_HINGE_SIZE = 0.07;
const BACK_HINGE_SIZE = 0.16;

function PointerGauge () {
	const backHinge = BACK_HINGE_SIZE,
		fontHinge = FONT_HINGE_SIZE;

	return [
		//backHinge,
		fontHinge
	]
}

export default PointerGauge;
