import __icon__$1 from './one.js';

const __icon__ = {};

var icons = /*#__PURE__*/Object.freeze({
	__proto__: null,
	one: __icon__$1,
	two: __icon__
});

const __component__ = { icons };

export { __component__ as Component_one };
