System.register(['./generated-shared.js', './generated-dep.js'], (function () {
	'use strict';
	return {
		setters: [function () {}, function () {}],
		execute: (function () {

			console.log('main2');

		})
	};
}));
