System.register([], (function (exports) {
	'use strict';
	return {
		execute: (function () {

			var num = exports('n', 2);

		})
	};
}));
console.log('fileName', 'chunk-dep2.js');
console.log('imports', '');
console.log('isEntry', false);
console.log('name', 'dep2');
console.log('modules.length', 1);
