'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var lib = require('./lib.js');

function _interopNamespaceDefaultOnly (e) { return Object.freeze({ __proto__: null, 'default': e }); }

var lib__namespace = /*#__PURE__*/_interopNamespaceDefaultOnly(lib);

console.log(lib__namespace);
Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespaceDefaultOnly(require('./lib.js')); }).then(console.log);

exports.lib = lib__namespace;
