define(['exports'], (function (exports) { 'use strict';

	var a = 'a';
	var b = 'a';

	var main2 = /*#__PURE__*/Object.freeze({
		__proto__: null,
		a: a,
		b: b
	});

	exports.a = a;
	exports.b = b;
	exports.main2 = main2;

}));
