System.register([], (function (exports, module) {
	'use strict';
	return {
		execute: (function () {

			var solved = exports('default', 'nested/chunk.js:solved:assets/asset-solved-28a7ac89.txt:../assets/asset-solved-28a7ac89.txt');

		})
	};
}));
