define(['external-true'], (function (externalTrue) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var externalTrue__default = /*#__PURE__*/_interopDefaultLegacy(externalTrue);



	return externalTrue__default["default"];

}));
