define(['exports', './one'], (function (exports, one) { 'use strict';

	const __icon__ = {};

	var icons = /*#__PURE__*/Object.freeze({
		__proto__: null,
		one: one,
		two: __icon__
	});

	const __component__ = { icons };

	exports.Component_one = __component__;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
