var myBundle = (function (exports) {
	'use strict';

	var dep$1 = 'js';

	var dep = 'mjs';

	exports.depJs = dep$1;
	exports.depMjs = dep;

	Object.defineProperty(exports, '__esModule', { value: true });

	return exports;

})({});
