define(['external1', 'external2'], (function (external1, external2) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var external2__default = /*#__PURE__*/_interopDefaultLegacy(external2);

	console.log(external1.foo);

	return external2__default["default"];

}));
