define(['exports'], (function (exports) { 'use strict';

	var num = 2;

	exports.num = num;

}));
console.log('fileName', 'chunk-dep2.js');
console.log('imports', '');
console.log('isEntry', false);
console.log('name', 'dep2');
console.log('modules.length', 1);
