define(['exports', './generated-otherEntry'], (function (exports, otherEntry) { 'use strict';



	exports.b = otherEntry.b;
	exports.c = otherEntry.c;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
