{
	console.log('retained');
}

{
	console.log('retained');
}

{
	console.log('retained');
}

{
	console.log('retained');
}

{
	console.log('retained');
}

{
	console.log('retained');
}

{
	console.log('retained');
}
