import asset1a from 'asset1a';
import asset1b from 'asset1b';
import asset2a from 'asset2a';
import asset2b from 'asset2b';
import asset99a from 'asset99a';

console.log(asset1a, asset1b, asset2a, asset2b, asset99a);
