System.register([], (function (exports) {
  'use strict';
  return {
    execute: (function () {

      var main = exports('default', typeof global !== "undefined" ? global :
        typeof self !== "undefined" ? self :
        typeof window !== "undefined" ? window : {});

    })
  };
}));
