'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

exports.x = 1;
exports.x = 2;
exports.x += 1;
exports.x -= 1;
exports.x *= 2;
exports.x /= 2;
exports.x %= 2;
exports.x **= 2;
exports.x <<= 1;
exports.x >>= 1;
exports.x >>>= 1;
exports.x &= 3;
exports.x ^= 2;
exports.x |= 2;
