console.log('dep');

const dep = 'dep';

console.log('dynamic', dep);
const dynamic = 'dynamic';

var dynamic$1 = /*#__PURE__*/Object.freeze({
	__proto__: null,
	dynamic: dynamic
});

export { dep as a, dynamic$1 as b, dynamic as d };
