define(['../../../_virtual/index.js_commonjs-exports'], (function (index) { 'use strict';

	Object.defineProperty(index.__exports, '__esModule', { value: true });

	var hello = 'world';

	index.__exports.hello = hello;

	return index.__exports;

}));
