var exposedInternals = (function (exports) {
	'use strict';

	const a = 1;
	const b = 2;

	exports.a = a;
	exports.b = b;

	Object.defineProperty(exports, '__esModule', { value: true });

	return exports;

})({});
