define(['leaflet'], (function (L) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var L__default = /*#__PURE__*/_interopDefaultLegacy(L);

	L__default["default"].terminator = function(options) {
	};

}));
