'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const bar = 2;
Promise.resolve().then(function () { return foo$1; });

const foo = 1;

var foo$1 = /*#__PURE__*/Object.freeze({
	__proto__: null,
	foo: foo
});

exports.bar = bar;
