'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function doThings() {
	console.log( 'doing things...' );
}

const number = 42;

var setting = 'no';

exports.doThings = doThings;
exports.number = number;
exports.setting = setting;
