'use strict';

function _interopNamespaceDefaultOnly (e) { return Object.freeze({ __proto__: null, [Symbol.toStringTag]: 'Module', 'default': e }); }

Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespaceDefaultOnly(require('./foo.js')); }).then(console.log);
