define(['exports', './generated-dep', 'external'], (function (exports, dep, external) { 'use strict';



	Object.defineProperty(exports, 'dep', {
		enumerable: true,
		get: function () { return external.asdf; }
	});

	Object.defineProperty(exports, '__esModule', { value: true });

}));
