import * as foo from 'external-all';
import * as bar from 'external-namespace';
import * as baz from 'external-default-namespace';
import * as quux from 'external-named-namespace';
console.log(foo, bar, baz, quux);
