define(['exports', './generated-lib'], (function (exports, lib) { 'use strict';



	exports.lib = lib.lib;
	exports.foo = lib.lib.foo;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
