(function (factory) {
	typeof define === 'function' && define.amd ? define(factory) :
	factory();
})((function () { 'use strict';

	const other$1 = {
		something: 'here'
	};

	const other = {
		somethingElse: 'here'
	};

}));
