(function (unchanged, changedName, specialCharacter, slash, relative_js) {
	'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var changedName__default = /*#__PURE__*/_interopDefaultLegacy(changedName);

	console.log(unchanged.foo, changedName__default["default"], specialCharacter.bar, slash.baz, relative_js.quux);

})(unchanged, changedName, specialCharacter, slash, relative_js);
