'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var one = require('./generated-one.js');

class Two {
    test() {
        return one.ONE_CONSTANT;
    }
}

exports.ItemTwo = Two;
