import * as externalFalse from 'external-false';
export { externalFalse };
export { barFalse, default as fooFalse } from 'external-false';
import * as externalTrue from 'external-true';
export { externalTrue };
export { barTrue, default as fooTrue } from 'external-true';
