define(['./commonjs', 'external'], (function (commonjs, require$$0) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var require$$0__default = /*#__PURE__*/_interopDefaultLegacy(require$$0);

	console.log(commonjs, require$$0__default["default"]);

}));
