define(['require', 'exports'], (function (require, exports) { 'use strict';

	var solved = 'nested/chunk.js:solved:assets/asset-solved-28a7ac89.txt:../assets/asset-solved-28a7ac89.txt';

	exports["default"] = solved;

}));
