'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var self = /*#__PURE__*/Object.freeze({
	__proto__: null,
	get p () { return p; }
});

console.log(Object.keys(self));

var p = 5;

exports.p = p;
