export

/* comment */
// aaaa

/* foo */

default // jsjjjsjjjjsjs

/* jsjjddjksj
*/ // Too many comments lol

async /* [no LineTerminator here] */ function

//iian iaouns
/* aaain ob skubz */

() {
  console.log("Foo");
}
