define(['exports', './chunks/shared'], (function (exports, shared) { 'use strict';



	exports.sharedValue = shared.sharedValue;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
