exports.amd = {"code":"define(function(){'use strict';console.log(42)});","map":{"version":3,"sources":["unknown"],"names":["define","console","log"],"mappings":"AAAAA,OAAO,UAAY,CAAE,aAErBC,QAAQC,GAAR,CAAa,EAAb,CAEC,CAJD,C","file":"unknown","sourcesContent":["define(function () { 'use strict';\n\nconsole.log( 42 );\n\n});"]}};
exports.cjs = {"code":"'use strict';console.log(42);","map":{"version":3,"sources":["unknown"],"names":["console","log"],"mappings":"AAAA,aAEAA,QAAQC,GAAR,CAAa,EAAb,C","file":"unknown","sourcesContent":["'use strict';\n\nconsole.log( 42 );"]}};
exports.es = {"code":"console.log(42);","map":{"version":3,"sources":["unknown"],"names":["console","log"],"mappings":"AAAAA,QAAQC,GAAR,CAAa,EAAb,C","file":"unknown","sourcesContent":["console.log( 42 );"]}};
exports.iife = {"code":"(function(){'use strict';console.log(42)})();","map":{"version":3,"sources":["unknown"],"names":["console","log"],"mappings":"AAAC,WAAY,CACb,aAEAA,QAAQC,GAAR,CAAa,EAAb,CAEC,CALA,G","file":"unknown","sourcesContent":["(function () {\n'use strict';\n\nconsole.log( 42 );\n\n}());"]}};
exports.umd = {"code":"(function(a){'function'==typeof define&&define.amd?define(a):a()})(function(){'use strict';console.log(42)});","map":{"version":3,"sources":["unknown"],"names":["define","amd","console","log"],"mappings":"AAAC,YAAmB,CACF,UAAlB,QAAOA,OAAP,EAAgCA,OAAOC,GAAvC,CAA6CD,SAA7C,CACA,GACC,CAHA,EAGC,UAAY,CAAE,aAEhBE,QAAQC,GAAR,CAAa,EAAb,CAEC,CAPA,C","file":"main.js","sourcesContent":["(function (factory) {\ntypeof define === 'function' && define.amd ? define(factory) :\nfactory();\n}(function () { 'use strict';\n\nconsole.log( 42 );\n\n}));\n"]}};
