'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var external = require('external');



exports.p = external;
Object.defineProperty(exports, 'q', {
	enumerable: true,
	get: function () { return external.p; }
});
