'use strict';

var externalTrue = require('external-true');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var externalTrue__default = /*#__PURE__*/_interopDefaultLegacy(externalTrue);



module.exports = externalTrue__default["default"];
