import { virtual } from './_virtual/entry-_virtualModule-es-.mjs';
import { virtual2 } from './_virtual/entry-_virtualWithExt-es-js.js.mjs';
import { virtual3 } from './_virtual/entry-_virtualWithAssetExt-es-str.str.str.mjs';

assert.equal(virtual, 'Virtual!');
assert.equal(virtual2, 'Virtual2!');
assert.equal(virtual3, 'Virtual3!');
