System.register(['./generated-dep11.js', './generated-dep111.js', './generated-dep112.js'], (function () {
	'use strict';
	return {
		setters: [function () {}, function () {}, function () {}],
		execute: (function () {



		})
	};
}));
