const localIsNaN = isNaN;
const localIsFinite = isFinite;
const localParseFloat = parseFloat;
const localNumber = Number;
export {
	localIsNaN as isNaN,
	localIsFinite as isFinite,
	localParseFloat as parseFloat,
	localNumber as Number
};
