define(['exports', './hsl2hsv'], (function (exports, hsl2hsv$1) { 'use strict';

	var hsl2hsv = 'asdf';

	console.log(hsl2hsv);

	var lib = /*#__PURE__*/Object.freeze({
		__proto__: null,
		hsl2hsv: hsl2hsv$1["default"]
	});

	exports.lib = lib;

}));
