System.register(['starexternal2', 'external2'], (function (exports) {
	'use strict';
	return {
		setters: [function () {}, function () {}],
		execute: (function () {

			var dep = exports('d', 'dep');

		})
	};
}));
