'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var something = require('./inner/more_inner/something.js');
require('./inner/some_effect.js');



exports.Something = something.Something;
