define(['exports'], (function (exports) { 'use strict';

	const localIsNaN = isNaN;

	const isNaN$1 = localIsNaN;

	exports.isNaN = isNaN$1;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
