import { Something } from './Something';
import { SomethingAgain } from './SomethingAgain';
import { Material } from './Material';
import { MaterialAgain } from './MaterialAgain';

var result = [Material(), MaterialAgain(), Something(), SomethingAgain()]
export default result;
