define(['module', 'exports'], (function (module, exports) { 'use strict';

	const url = 'url';
	const meta = 'meta';

	exports.meta = meta;
	exports.url = url;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
