if ((1 + 1).unknown) {
	console.log('retained');
} else {
	console.log('retained');
}
console.log('retained 1');
console.log('retained 2');
console.log('retained 3');
console.log('retained 4');
console.log('retained 5');
console.log('retained 6');
console.log('retained 7');
console.log('retained 8');
console.log('retained 9');
console.log('retained 10');
console.log('retained 11');
console.log('retained 12');
console.log('retained 13');
console.log('retained 14');
console.log('retained 15');
console.log('retained 16');
console.log('retained 17');
if (1 in 2) console.log('retained 18');
if (1 instanceof 2) console.log('retained 19');
console.log('retained 20');
