define(['exports', './generated-main'], (function (exports, main) { 'use strict';



	exports.dynamic = main.dynamic;
	exports.unused = main.unused;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
