define(['external'], (function (ns) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

	var ns__default = /*#__PURE__*/_interopDefaultLegacy(ns);

	console.log(ns__default);

}));
