'use strict';

var quoted_____external1 = require('quoted\'\
\
\ \ external1');
var quoted_____external2 = require('./quoted\'\
\
\ \ external2');
var Path_js = require('./C:/File/Path.js');

console.log(quoted_____external1.foo, quoted_____external2.bar, Path_js.baz);
