// @ts-ignore
export { default as foo } from './foo.ts';
// @ts-ignore
export { default as bar } from './nested/bar.ts';
// @ts-ignore
export { default as baz } from './nested/baz.ts';
// @ts-ignore
export { default as lorem } from './lorem.str';
// @ts-ignore
export { default as noExt } from './no-ext';
