define(['exports', './generated-build-starter'], (function (exports, buildStarter) { 'use strict';



	exports.buildStartValue = buildStarter.value;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
