var libraryName = (function (exports) {
	'use strict';

	const valueOnLib = 42;

	exports.valueOnLib = valueOnLib;

	Object.defineProperty(exports, '__esModule', { value: true });

	return exports;

})({});
