define(['exports', 'external'], (function (exports, external) { 'use strict';



	Object.defineProperty(exports, 's', {
		enumerable: true,
		get: function () { return external.p; }
	});

	Object.defineProperty(exports, '__esModule', { value: true });

}));
