'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var one = require('./one.js');

const __icon__ = {};

var icons = /*#__PURE__*/Object.freeze({
	__proto__: null,
	one: one,
	two: __icon__
});

const __component__ = { icons };

exports.Component_one = __component__;
