'use strict';

var index = require('../../../_virtual/index.js_commonjs-exports.js');

Object.defineProperty(index.__exports, '__esModule', { value: true });

var hello = 'world';

index.__exports.hello = hello;

module.exports = index.__exports;
