import './custom_modules/@my-scope/my-base-pkg/index.js';
import { __exports as myBasePkg } from './_virtual/index.js_commonjs-exports.js';

const base = myBasePkg;

var underBuild = {
	base
};

export { underBuild as default };
