define(['external-auto'], (function (externalAuto) { 'use strict';

	function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

	var externalAuto__default = /*#__PURE__*/_interopDefault(externalAuto);



	return externalAuto__default["default"];

}));
