define(['exports', './generated-main1'], (function (exports, main2) { 'use strict';



	exports.p = main2.p;
	exports.p2 = main2.p$1;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
