define(['exports', './inner/more_inner/something', './inner/some_effect'], (function (exports, something, some_effect) { 'use strict';



	exports.Something = something.Something;

	Object.defineProperty(exports, '__esModule', { value: true });

}));
