define(['exports'], (function (exports) { 'use strict';

	const value = 42;
	const otherValue = 43;

	console.log('startBuild', value);

	exports.otherValue = otherValue;
	exports.value = value;

}));
