'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var index = require('./components/index.js');
var index$1 = require('./components/sub/index.js');



exports.baz = index.baz;
exports.foo = index$1.foo;
