import { r as reexported$1 } from './generated-dep.js';
import { reexported } from 'external';

console.log(reexported);

var lib = /*#__PURE__*/Object.freeze({
	__proto__: null,
	reexported: reexported$1
});

export { lib as l };
