var reexportsAliasingExternal = (function (exports, d) {
	'use strict';



	Object.defineProperty(exports, 'b', {
		enumerable: true,
		get: function () { return d.d; }
	});

	Object.defineProperty(exports, '__esModule', { value: true });

	return exports;

})({}, d);
