const foo = 'foo';

var volume = /*#__PURE__*/Object.freeze({
	__proto__: null,
	foo: foo
});

const bar = 'bar';

var geometry = /*#__PURE__*/Object.freeze({
	__proto__: null,
	bar: bar
});

export { bar as b, foo as f, geometry as g, volume as v };
