const foo1 = [1, 2, 3];
console.log(foo1[0]);

const foo2 = [1, 2, 3];
console.log(foo2[0]);

const foo3 = [1, 2, 3];
console.log(foo3[0]);

const foo4 = [1, 2, 3];
console.log(foo4[0]);

const foo5 = [1, 2, 3];
console.log(foo5[0]);

const foo6 = [1, 2, 3];
console.log(foo6[0]);

const foo7 = [1, 2, 3];
console.log(foo7[0]);

const foo8 = [1, 2, 3];
console.log(foo8[0]);

const foo9 = [1, 2, 3];
console.log(foo9[0]);

const foo10 = [1, 2, 3];
console.log(foo10[0]);

const foo11 = [1, 2, 3];
console.log(foo11[0]);

const foo12 = [1, 2, 3];
console.log(foo12[0]);

const foo13 = [1, 2, 3];
console.log(foo13[0]);

const foo14 = [1, 2, 3];
console.log(foo14[0]);
