System.register([], (function (exports, module) {
	'use strict';
	return {
		execute: (function () {

			var asset1a = new URL('assets/asset-dc5cb674', module.meta.url).href;

			var asset1b = new URL('assets/asset-dc5cb674', module.meta.url).href;

			var asset2a = new URL('assets/asset-52cbd095', module.meta.url).href;

			var asset2b = new URL('assets/asset-52cbd095', module.meta.url).href;

			var asset99a = new URL('assets/asset-c568a840', module.meta.url).href;

			console.log(asset1a, asset1b, asset2a, asset2b, asset99a);

		})
	};
}));
