const renderStart = new URL('assets/renderStart-66600c78.txt', import.meta.url).href;
const renderStartNamed = new URL('renderStart.txt', import.meta.url).href;
const renderStartNamedImmediately = 'renderStart.txt';
const banner = new URL('assets/banner-2b65cc0c.txt', import.meta.url).href;
const bannerNamed = new URL('banner.txt', import.meta.url).href;
const footer = new URL('assets/footer-e1d2fccb.txt', import.meta.url).href;
const footerNamed = new URL('footer.txt', import.meta.url).href;
const intro = new URL('assets/intro-520a8116.txt', import.meta.url).href;
const introNamed = new URL('intro.txt', import.meta.url).href;
const outro = new URL('assets/outro-21f77720.txt', import.meta.url).href;
const outroNamed = new URL('outro.txt', import.meta.url).href;

export { banner, bannerNamed, footer, footerNamed, intro, introNamed, outro, outroNamed, renderStart, renderStartNamed, renderStartNamedImmediately };
