export {default as resolvedString} from 'resolve-string';
export {default as resolvedExternal} from 'resolve-external';
export {default as resolvedObject} from 'resolve-object';
export {default as resolvedObjectExternal} from 'resolve-object-external';
