/* this is an intro */

// intro 1

// intro 2

// intro 3

// intro 4

import a, { b } from 'external';

console.log( a );
console.log( b );

var main = 42;

export { main as default };

/* this is an outro */

// outro 1

// outro 2

// outro 3

// outro 4
